@@ -8,6 +8,8 @@ This module contains all classes related to metaTOR commands:
     - partition
     - pipeline
     - validation
+    - qualitycheck
+    - contactmap
 
 Note
 ----
@@ -22,18 +24,22 @@ NotImplementedError
     of its children.
 """
 
+import logging
 import os
 import shutil
 import time
 import metator.align as mta
+import metator.contact_map as mtc
 import metator.io as mio
 import metator.log as mtl
+import metator.quality_check as mtq
 import metator.network as mtn
 import metator.partition as mtp
+import metator.scaffold as mts
 import metator.validation as mtv
-import metator.contact_map as mtc
 from docopt import docopt
 from metator.log import logger
+from metator.version import __version__
 from os.path import exists, dirname, join
 from scipy.sparse import save_npz, load_npz
 
@@ -85,7 +91,7 @@ class Network(AbstractCommand):
 
     usage:
         network --forward=STR --assembly=FILE [--reverse=STR]
-        [--aligner=bowtie2] [--aligner-mode=normal] [--depth=FILE]
+        [--aligner=bowtie2] [--aligner-mode=normal] [--depth=FILE] [--edge=0]
         [--enzyme=STR] [--normalization=empirical_hit] [--no-clean-up]
         [--outdir=DIR] [--min-quality=30] [--self-contacts] [--start=fastq]
         [--threads=1] [--tmpdir=DIR]
@@ -111,6 +117,9 @@ class Network(AbstractCommand):
                                 to made the assembly computed by
                                 jgi_summarize_bam_contig_depths from metabat2
                                 pipeline.
+        -E, --edge=INT          Distance of the edge region in base pair on the
+                                contigs where the mapping reads are not
+                                considered as inter contigs. [Default: 0]
         -e, --enzyme=STR        The list of restriction enzyme used to digest
                                 the contigs separated by a comma. Example:
                                 HpaII,MluCI.
@@ -152,13 +161,12 @@ class Network(AbstractCommand):
 
         # Enable file logging
         now = time.strftime("%Y%m%d%H%M%S")
-        log_file = join(
-            self.args["--outdir"], ("metator_network_" + now + ".log")
-        )
-        mtl.set_file_handler(log_file)
+        log_file = join(self.args["--outdir"], (f"metator_network_{now}.log"))
+        generate_log_header(log_file, cmd="network", args=self.args)
 
         # Transform integer variables as integer.
         min_qual = int(self.args["--min-quality"])
+        edge = int(self.args["--edge"])
 
         # Defined boolean variables:
         self_contacts = self.args["--self-contacts"]
@@ -181,16 +189,18 @@ class Network(AbstractCommand):
         # Check aligner.
         if self.args["--aligner"] not in ["bowtie2", "bwa"]:
             logger.error('Aligner should be either "bowtie2" or "bwa".')
+            raise ValueError
 
         # Check aligner mode.
         if self.args["--aligner-mode"] not in [
             "normal",
             "iterative",
-            "cutsiste",
+            "cutsite",
         ]:
             logger.error(
                 'Aligner mode should be either "normal", "iterative" or "cutsite".'
             )
+            raise ValueError
         if (
             self.args["--aligner-mode"] == "cutsite"
             and not self.args["--enzyme"]
@@ -300,6 +310,7 @@ class Network(AbstractCommand):
         mtn.alignment_to_contacts(
             alignment_files,
             contig_data,
+            edge,
             hit_data,
             self.args["--outdir"],
             "network.txt",
@@ -314,6 +325,8 @@ class Network(AbstractCommand):
         if not self.args["--no-clean-up"]:
             shutil.rmtree(tmp_dir)
 
+        generate_log_footer(log_file)
+
 
 class Partition(AbstractCommand):
     """Partition the network using Louvain algorithm
@@ -402,10 +415,8 @@ class Partition(AbstractCommand):
 
         # Enable file logging
         now = time.strftime("%Y%m%d%H%M%S")
-        log_file = join(
-            self.args["--outdir"], ("metator_partition_" + now + ".log")
-        )
-        mtl.set_file_handler(log_file)
+        log_file = join(self.args["--outdir"], (f"metator_partition_{now}.log"))
+        generate_log_header(log_file, cmd="partition", args=self.args)
 
         # Transform numeric variable as numeric
         iterations = int(self.args["--iterations"])
@@ -442,6 +453,8 @@ class Partition(AbstractCommand):
         if not self.args["--no-clean-up"]:
             shutil.rmtree(tmp_dir)
 
+        generate_log_footer(log_file)
+
 
 class Validation(AbstractCommand):
     """Use CheckM to validate the bins.
@@ -464,8 +477,8 @@ class Validation(AbstractCommand):
     options:
         -a, --assembly=FILE     The path to the assembly fasta file used to do
                                 the alignment.
-        -A, --algorithm=STR     Algorithm to use. Either "louvain" or "leiden".
-                                [Default: louvain]
+        -A, --algorithm=STR     Algorithm to use. Either "louvain", "leiden" or
+                                "spinglass". [Default: louvain]
         -c, --contigs=FILE      The path to the file containing the data of the
                                 contigs from the partition step (13 columns).
         -C, --cluster-matrix    If enabled, save the clustering matrix.
@@ -540,9 +553,9 @@ class Validation(AbstractCommand):
         # Enable file logging
         now = time.strftime("%Y%m%d%H%M%S")
         log_file = join(
-            self.args["--outdir"], ("metator_validation_" + now + ".log")
+            self.args["--outdir"], (f"metator_validation_{now}.log")
         )
-        mtl.set_file_handler(log_file)
+        generate_log_header(log_file, cmd="validation", args=self.args)
 
         # Transform numeric variable as numeric
         iterations = int(self.args["--iterations"])
@@ -551,16 +564,11 @@ class Validation(AbstractCommand):
         overlapping_parameter = int(self.args["--overlap"]) / 100
         resolution_parameter = float(self.args["--res-param"])
 
-        # Check checkM availabilitys
-        if not mio.check_checkm():
-            logger.error(
-                "CheckM is not in the path. Could not make the iterations"
-            )
-            raise NameError
-
         # Check correct algorithm value
-        if self.args["--algorithm"] not in ["louvain", "leiden"]:
-            logger.error('algorithm should be either "louvain" or "leiden"')
+        if self.args["--algorithm"] not in ["louvain", "leiden", "spinglass"]:
+            logger.error(
+                'algorithm should be either "louvain", "leiden" or "spinglass.'
+            )
             raise ValueError
 
         _clustering_matrix_file = mtv.recursive_decontamination(
@@ -587,6 +595,8 @@ class Validation(AbstractCommand):
         if not self.args["--no-clean-up"]:
             shutil.rmtree(tmp_dir)
 
+        generate_log_footer(log_file)
+
 
 class Pipeline(AbstractCommand):
     """Launch the full metator pipeline
@@ -601,12 +611,12 @@ class Pipeline(AbstractCommand):
     usage:
         pipeline --assembly=FILE [--forward=STR] [--reverse=STR]
         [--algorithm=louvain] [--aligner=bowtie2] [--aligner-mode=normal]
-        [--cluster-matrix] [--contigs=FILE] [--depth=FILE] [--enzyme=STR]
-        [--force] [--iterations=100] [--rec-iter=10] [--network=FILE]
-        [--no-clean-up] [--normalization=empirical_hit] [--outdir=DIR]
-        [--overlap=80] [--rec-overlap=90]  [--min-quality=30] [--res-param=1.0]
+        [--cluster-matrix] [--contigs=FILE] [--depth=FILE] [--edge=0]
+        [--enzyme=STR] [--force] [--iterations=100] [--rec-iter=10]
+        [--junctions=NNNNN] [--network=FILE] [--no-clean-up]
+        [--normalization=empirical_hit] [--outdir=DIR] [--overlap=80]
+        [--rec-overlap=90]  [--min-quality=30] [--res-param=1.0]
         [--size=500000] [--start=fastq] [--threads=1] [--tmpdir=DIR]
-        [--skip-validation]
 
     options:
         -1, --forward=STR       Fastq file or list of Fastq separated by a comma
@@ -621,7 +631,9 @@ class Pipeline(AbstractCommand):
         -a, --assembly=FILE     The initial assembly path acting as the
                                 alignment file's reference genome or the
                                 basename of the bowtie2 index.
-        -A, --algorithm=STR     Algorithm to use. Either "louvain" or "leiden".
+        -A, --algorithm=STR     Algorithm to use. Either "louvain", "leiden" or
+                                "spinglass". If spinglass is chosen, the first
+                                partition will be done using louvain
                                 [Default: louvain]
         -b, --aligner=STR       Aligner algorithm to use. Either "bwa" or
                                 "bowtie2". [Default: bowtie2]
@@ -636,6 +648,9 @@ class Pipeline(AbstractCommand):
                                 to made the assembly computed by
                                 jgi_summarize_bam_contig_depths from metabat2
                                 pipeline.
+        -E, --edge=INT          Distance of the edge region in base pair on the
+                                contigs where the mapping reads are not
+                                considered as inter contigs. [Default: 0]
         -e, --enzyme=STR        The list of restriction enzyme used to digest
                                 the contigs separated by a comma. Example:
                                 HpaII,MluCI.
@@ -645,6 +660,8 @@ class Pipeline(AbstractCommand):
                                 partition step. [Default: 100]
         -j, --rec-iter=INT      Number of iterations of Louvain for the
                                 recursive step. [Default: 10]
+        -J, --junctions=STR     Sequences to use as junction between contigs.
+                                [Default: NNNNN]
         -n, --network=FILE      Path to the file containing the network
                                 information from the meta HiC experiment compute
                                 in network function previously.
@@ -675,8 +692,6 @@ class Pipeline(AbstractCommand):
                                 alignement. [Default: 1]
         -T, --tmpdir=DIR       Temporary directory. Default to current
                                 directory. [Default: ./tmp]
-        -v, --skip-validation   If  enables do not do the validation step which
-                                have an high memory usage (checkM ~ 40G)
     """
 
     def execute(self):
@@ -709,12 +724,13 @@ class Pipeline(AbstractCommand):
 
         # Enable file logging
         now = time.strftime("%Y%m%d%H%M%S")
-        log_file = join(self.args["--outdir"], ("metator_" + now + ".log"))
-        mtl.set_file_handler(log_file)
+        log_file = join(self.args["--outdir"], (f"metator_pipeline_{now}.log"))
+        generate_log_header(log_file, cmd="pipeline", args=self.args)
 
         # Define variable
         min_qual = int(self.args["--min-quality"])
         iterations = int(self.args["--iterations"])
+        edge = int(self.args["--edge"])
         recursive_iterations = int(self.args["--rec-iter"])
         overlapping_parameter = int(self.args["--overlap"]) / 100
         recursive_overlapping_parameter = int(self.args["--rec-overlap"]) / 100
@@ -725,16 +741,18 @@ class Pipeline(AbstractCommand):
         # Check aligner.
         if self.args["--aligner"] not in ["bowtie2", "bwa"]:
             logger.error('Aligner should be either "bowtie2" or "bwa".')
+            raise ValueError
 
         # Check aligner mode.
         if self.args["--aligner-mode"] not in [
             "normal",
             "iterative",
-            "cutsiste",
+            "cutsite",
         ]:
             logger.error(
                 'Aligner mode should be either "normal", "iterative" or "cutsite".'
             )
+            raise ValueError
         if (
             self.args["--aligner-mode"] == "cutsite"
             and not self.args["--enzyme"]
@@ -745,8 +763,10 @@ class Pipeline(AbstractCommand):
             self.args["--aligner-mode"] = "iterative"
 
         # Check correct algorithm value.
-        if self.args["--algorithm"] not in ["louvain", "leiden"]:
-            logger.error('algorithm should be either "louvain" or "leiden"')
+        if self.args["--algorithm"] not in ["louvain", "leiden", "spinglass"]:
+            logger.error(
+                'algorithm should be either "louvain", "leiden" or "spinglass.'
+            )
             raise ValueError
 
         # Check if normalization in the list of possible normalization.
@@ -783,40 +803,47 @@ class Pipeline(AbstractCommand):
             raise ValueError
 
         # Sanity check for validation
-        if not self.args["--skip-validation"]:
-            recursive_fasta_dir = join(self.args["--outdir"], "recursive_bin")
-            if not exists(recursive_fasta_dir):
+        recursive_fasta_dir = join(self.args["--outdir"], "recursive_bin")
+        if not exists(recursive_fasta_dir):
+            os.makedirs(recursive_fasta_dir)
+        else:
+            if self.args["--force"]:
+                shutil.rmtree(recursive_fasta_dir)
                 os.makedirs(recursive_fasta_dir)
             else:
-                if self.args["--force"]:
-                    shutil.rmtree(recursive_fasta_dir)
-                    os.makedirs(recursive_fasta_dir)
-                else:
-                    logger.error(
-                        "%s already existed. Remove directory or use -F argument to overwrite it",
-                        recursive_fasta_dir,
-                    )
-                    raise ValueError
-            final_fasta_dir = join(self.args["--outdir"], "final_bin")
-            if not exists(final_fasta_dir):
+                logger.error(
+                    "%s already existed. Remove directory or use -F argument to overwrite it",
+                    recursive_fasta_dir,
+                )
+                raise ValueError
+        final_fasta_dir = join(self.args["--outdir"], "final_bin_unscaffold")
+        if not exists(final_fasta_dir):
+            os.makedirs(final_fasta_dir)
+        else:
+            if self.args["--force"]:
+                shutil.rmtree(final_fasta_dir)
                 os.makedirs(final_fasta_dir)
             else:
-                if self.args["--force"]:
-                    shutil.rmtree(final_fasta_dir)
-                    os.makedirs(final_fasta_dir)
-                else:
-                    logger.error(
-                        "%s already existed. Remove directory or use -F argument to overwrite it.",
-                        final_fasta_dir,
-                    )
-                    raise ValueError
-
-            # Check checkM availability
-            if not mio.check_checkm():
                 logger.error(
-                    "CheckM is not in the path. Could not make the iterations"
+                    "%s already existed. Remove directory or use -F argument to overwrite it.",
+                    final_fasta_dir,
                 )
-                raise NameError
+                raise ValueError
+
+        # Sanity check for scaffolding
+        scaffold_fasta_dir = join(self.args["--outdir"], "scaffold_bin")
+        if not exists(scaffold_fasta_dir):
+            os.makedirs(scaffold_fasta_dir)
+        else:
+            if self.args["--force"]:
+                shutil.rmtree(scaffold_fasta_dir)
+                os.makedirs(scaffold_fasta_dir)
+            else:
+                logger.error(
+                    "%s already existed. Remove directory or use -F argument to overwrite it.",
+                    scaffold_fasta_dir,
+                )
+                raise ValueError
 
         # Manage start point.
         if self.args["--start"] == "fastq":
@@ -850,22 +877,18 @@ class Pipeline(AbstractCommand):
 
         # Print information of the workflow:
         if start == 1:
-            logger.info("Minimum mapping quality: %d", min_qual)
+            logger.info(f"Minimum mapping quality: {min_qual}")
         if start <= 2:
-            logger.info("Enzyme: %s", self.args["--enzyme"])
-            logger.info("Normalization: %s", self.args["--normalization"])
-        logger.info("Aligner algorithm: %s", self.args["--aligner"])
-        logger.info("Partition algorithm: %s", self.args["--algorithm"])
-        logger.info("Partition iterations: %s", iterations)
-        logger.info("Overlapping parameter: %s", overlapping_parameter)
-        if not self.args["--skip-validation"]:
-            logger.info(
-                "Recursive partition iterations: %d", recursive_iterations
-            )
-            logger.info(
-                "Recursive overlapping parameter: %s",
-                recursive_overlapping_parameter,
-            )
+            logger.info(f"Enzyme: {self.args['--enzyme']}")
+            logger.info(f"Normalization: {self.args['--normalization']}")
+        logger.info(f"Aligner algorithm: {self.args['--aligner']}")
+        logger.info(f"Partition algorithm: {self.args['--algorithm']}")
+        logger.info(f"Partition iterations: {iterations}")
+        logger.info(f"Overlapping parameter: {overlapping_parameter}")
+        logger.info(f"Recursive partition iterations: {recursive_iterations}")
+        logger.info(
+            f"Recursive overlapping parameter: {recursive_overlapping_parameter}\n"
+        )
 
         # Extract index and genome file
         assembly = self.args["--assembly"]
@@ -919,6 +942,7 @@ class Pipeline(AbstractCommand):
             network_file, contigs_data_file = mtn.alignment_to_contacts(
                 alignment_files,
                 contig_data,
+                edge,
                 hit_data,
                 self.args["--outdir"],
                 "network.txt",
@@ -956,53 +980,213 @@ class Pipeline(AbstractCommand):
             )
             os.remove(contig_data_network_file)
 
-        # Launch validation if desired.
-        if not self.args["--skip-validation"]:
-            clustering_matrix_recursive_file = mtv.recursive_decontamination(
-                self.args["--algorithm"],
-                fasta,
-                self.args["--cluster-matrix"],
-                contigs_data_file,
-                final_fasta_dir,
-                overlapping_fasta_dir,
-                recursive_iterations,
-                network_file,
-                self.args["--outdir"],
-                recursive_overlapping_parameter,
-                recursive_fasta_dir,
-                resolution_parameter,
-                size,
-                tmp_dir,
-                threads,
-            )
+        # Launch validation.
+        clustering_matrix_recursive_file = mtv.recursive_decontamination(
+            self.args["--algorithm"],
+            fasta,
+            self.args["--cluster-matrix"],
+            contigs_data_file,
+            final_fasta_dir,
+            overlapping_fasta_dir,
+            recursive_iterations,
+            network_file,
+            self.args["--outdir"],
+            recursive_overlapping_parameter,
+            recursive_fasta_dir,
+            resolution_parameter,
+            size,
+            tmp_dir,
+            threads,
+        )
 
-            if self.args["--cluster-matrix"]:
-                # Make the sum with the partiton clustering matrix and save it.
-                clustering_matrix = load_npz(
-                    clustering_matrix_partition_file + ".npz"
-                )
-                clustering_matrix_recursive = load_npz(
-                    clustering_matrix_recursive_file + ".npz"
-                )
-                clustering_matrix = (
-                    (clustering_matrix + clustering_matrix_recursive) / 2
-                ).tocoo()
-                clustering_matrix_file = join(
-                    self.args["--outdir"], "clustering_matrix"
-                )
-                save_npz(clustering_matrix_file, clustering_matrix)
+        if self.args["--cluster-matrix"]:
+            # Make the sum with the partiton clustering matrix and save it.
+            clustering_matrix = load_npz(
+                clustering_matrix_partition_file + ".npz"
+            )
+            clustering_matrix_recursive = load_npz(
+                clustering_matrix_recursive_file + ".npz"
+            )
+            clustering_matrix = (
+                (clustering_matrix + clustering_matrix_recursive) / 2
+            ).tocoo()
+            clustering_matrix_file = join(
+                self.args["--outdir"], "clustering_matrix"
+            )
+            save_npz(clustering_matrix_file, clustering_matrix)
 
-            # Remove contig_data_partition file
-            contig_data_partition_file = join(
-                self.args["--outdir"], "contig_data_partition.txt"
+        # Remove contig_data_partition file
+        contig_data_partition_file = join(
+            self.args["--outdir"], "contig_data_partition.txt"
+        )
+        os.remove(contig_data_partition_file)
+
+        # Launch the scaffold
+        bin_summary = mio.read_bin_summary(
+            join(self.args["--outdir"], "bin_summary.txt")
+        )
+        for bin_name in bin_summary.index:
+            mts.get_scaffolds(
+                bin_name,
+                join(final_fasta_dir, f"{bin_name}.fa"),
+                alignment_files,
+                join(scaffold_fasta_dir, f"{bin_name}_scaffolded.fa"),
+                join(scaffold_fasta_dir, f"{bin_name}_info_frags.txt"),
+                threshold=0.05,
+                threads=int(self.args["--threads"]),
+                window_size=5000,
+                junctions=self.args["--junctions"],
             )
-            os.remove(contig_data_partition_file)
 
         # Delete pyfastx index:
         os.remove(fasta + ".fxi")
         # Delete the temporary folder.
         if not self.args["--no-clean-up"]:
             shutil.rmtree(tmp_dir)
+            if start < 3:
+                for pair_file in alignment_files:
+                    os.remove(pair_file)
+
+        generate_log_footer(log_file)
+
+
+class Qc(AbstractCommand):
+    """Generates some quality check on the output of metator.
+
+    The quality check is done on the high quality MAGs and only contigs witha
+    size bigger than 100kb. The goal is to assess HiC quality metrics on genomes
+    we kinda know to have a estimation of the quality of the library.
+
+    If the plot are set it will return some plot of the event distribution on
+    the high quality contigs.
+
+    usage:
+        qc --assembly=FILE --enzyme=STR [--bin-summary=FILE]
+        [--contig-data=FILE] [--metator-dir=DIR] [--no-clean-up] [--outdir=DIR]
+        [--pairs=FILE] [--prefix=STR] [--plot] [--threshold=STR] [--tmpdir=DIR]
+
+    options:
+        -a, --assembly=FILE     Path to the fasta file containing the contigs of
+                                interest. Could be the whole or the extracted
+                                contigs of one bin.
+        -b, --bin-summary=FILE  Path to the bin_summary.txt file from MetaTOR
+                                output.
+        -c, --contig-data=FILE  Path to the contig_data_final.txt file from
+                                MetaTOR output.
+        -e, --enzyme=STR        The list of restriction enzyme used to digest
+                                the contigs separated by a comma. Example:
+                                HpaII,MluCI.
+        -n, --no-clean-up       If enabled intermediary files will be kept.
+        -O, --metator-dir=DIR   Output directory from metator pipeline. If set
+                                contigs data and bin summary will be found
+                                automatically.
+        -o, --outdir=DIR        Directory to save output plots and log.
+        -p, --prefix=STR        Name of the sample to add on plot and files.
+        -P, --plot              If enable display some plots.
+        -r, --pairs=FILE        Path of the ".pairs" file. If more than one is
+                                given, files should be separated by a comma.
+        -t, --threshold=STR     Hicstuff religation and loop thresholds.
+                                Two integers seperated by a coma.
+        -T, --tmpdir=DIR        Temporary directory to save pairs files
+    """
+
+    def execute(self):
+
+        # Defined the temporary directory.
+        if not self.args["--tmpdir"]:
+            self.args["--tmpdir"] = "./tmp"
+        tmp_dir = mio.generate_temp_dir(self.args["--tmpdir"])
+
+        # Set output directory.
+        if not self.args["--outdir"]:
+            if self.args["--prefix"]:
+                prefix = self.args["--prefix"]
+                self.args["--outdir"] = join(".", f"metator_qc_{prefix}")
+            else:
+                self.args["--outdir"] = join(".", "metator_qc")
+        os.makedirs(self.args["--outdir"], exist_ok=True)
+
+        # Enable file logging
+        now = time.strftime("%Y%m%d%H%M%S")
+        log_file = join(self.args["--outdir"], (f"metator_qc_{now}.log"))
+        generate_log_header(log_file, cmd="qc", args=self.args)
+
+        # Try to find input files.
+        if self.args["--metator-dir"]:
+            metator_dir = self.args["--metator-dir"]
+        else:
+            metator_dir = "."
+        if self.args["--pairs"]:
+            pairs_files = self.args["--pairs"].split(",")
+        else:
+            pairs_files = [
+                join(metator_dir, file)
+                for file in filter(
+                    lambda x: "pairs" in x, os.listdir(metator_dir)
+                )
+            ]
+            if len(pairs_files) == 0:
+                logger.error(
+                    "Please give a pairs file, or the ouput directory of metator, or launch this command from the output directory of metator."
+                )
+                logger.error(
+                    "You can also check that a pairs file is present in metator output directory"
+                )
+                raise FileNotFoundError
+        if self.args["--bin-summary"]:
+            bin_summary_file = self.args["--bin-summary"]
+        else:
+            bin_summary_file = join(metator_dir, "bin_summary.txt")
+            if not exists(bin_summary_file):
+                logger.error(
+                    "Please give a bin summary file, or the ouput directory of metator or launch this command from the output directory of metator."
+                )
+                logger.error(
+                    "You can also check that a bin_summary.txt is present in metator output directory"
+                )
+                raise FileNotFoundError
+        if self.args["--contig-data"]:
+            contig_data_file = self.args["--contig-data"]
+        else:
+            contig_data_file = join(metator_dir, "contig_data_final.txt")
+            if not exists(contig_data_file):
+                logger.error(
+                    "Please give a contig data file, or the ouput directory of metator or launch this command from the output directory of metator."
+                )
+                logger.error(
+                    "You can also check that a contig_data_final.txt is present in metator output directory"
+                )
+                raise FileNotFoundError
+
+        # Set parameters.
+        if self.args["--prefix"]:
+            prefix = self.args["--prefix"]
+        else:
+            prefix = "metator_qc"
+        if self.args["--enzyme"]:
+            self.args["--enzyme"] = self.args["--enzyme"].split(",")
+        if self.args["--threshold"]:
+            self.args["--threshold"] = self.args["--threshold"].split(",")
+
+        # Launch quality check
+        mtq.quality_check(
+            contig_data_file,
+            bin_summary_file,
+            self.args["--assembly"],
+            pairs_files,
+            self.args["--outdir"],
+            tmp_dir,
+            prefix,
+            self.args["--plot"],
+            enzyme=self.args["--enzyme"],
+            threshold=self.args["--threshold"],
+        )
+
+        # Delete the temporary folder.
+        if not self.args["--no-clean-up"]:
+            shutil.rmtree(tmp_dir)
+
+        generate_log_footer(log_file)
 
 
 class Contactmap(AbstractCommand):
@@ -1029,7 +1213,7 @@ class Contactmap(AbstractCommand):
         -a, --assembly=FILE     Path to the fasta file containing the contigs of
                                 interest. Could be the whole or the extracted
                                 contigs of one bin.
-        -c, --contig-data=FILE  Path to the contig_data_final.txt file form
+        -c, --contig-data=FILE  Path to the contig_data_final.txt file from
                                 MetaTOR output.
         -e, --enzyme=STR        The list of restriction enzyme used to digest
                                 the contigs separated by a comma. Example:
@@ -1102,3 +1286,124 @@ class Contactmap(AbstractCommand):
             shutil.rmtree(tmp_dir)
             # Delete pyfastx index:
             os.remove(self.args["--assembly"] + ".fxi")
+
+
+class Scaffold(AbstractCommand):
+    """Scaffold a bin from metator.
+
+    Use the pairs to build a scaffold from the binned contigs. The scaffolding
+    is based on contact between the edges of the contigs. There are normalized
+    based on intra contacts of the contigs except for the small contigs where
+    a low score is given.
+
+    usage:
+        scaffold --bin-name=STR --input-fasta=FILE [--junctions=NNNNN]
+        [--out-fasta=FILE] [--out-frags=FILE] [--threads=1] [--threshold=0.1]
+        [--window-size=5000] <pairsfile>...
+
+    arguments:
+        pairsfile               File(s) containing pairs information.
+
+    options:
+        -b, --bin-name=STR      Name of the bin to scaffold.
+        -i, --input-fasta=FILE  Path to the input fasta.
+        -j, --junctions=STR     Sequences to use as junction between contigs.
+                                [Default: NNNNN]
+        -o, --out-fasta=FILE    Path to write the output fasta. Default in the
+                                current directory: '"$bin_name"_scaffolded.fa'.
+        -O, --out-frags=FILE    Path to write the fragments information. Default
+                                in the current directory:
+                                '"$bin_name"_info_frags.txt'.
+        -t, --threads=INT       Numbers of threads to allocate if pairs need to
+                                be sorted. [Default: 1]
+        -T, --threshold=FLOAT   Threshold score to consider an association
+                                between two contigs. [Default 0.1]
+        -w, --window-size=INT   Size of the window in base pair to use as edge
+                                window to scaffold. [Default: 5000]
+    """
+
+    def execute(self):
+        # Generate log
+        now = time.strftime("%Y%m%d%H%M%S")
+        log_file = f"metator_scaffold_{now}.log"
+        generate_log_header(log_file, cmd="scaffold", args=self.args)
+
+        # Create output files:
+        if self.args["--out-fasta"] is None:
+            out_fasta = join(".", f'{self.args["--bin-name"]}_scaffolded.fa')
+        else:
+            out_fasta = self.args["--out-fasta"]
+        if self.args["--out-frags"] is None:
+            out_info = join(".", f'{self.args["--bin-name"]}_info_frags.txt')
+        else:
+            out_info = self.args["--out-frags"]
+
+        # Get the scaffolds
+        mts.get_scaffolds(
+            self.args["--bin-name"],
+            self.args["--input-fasta"],
+            self.args["<pairsfile>"],
+            out_fasta,
+            out_info,
+            threshold=float(self.args["--threshold"]),
+            threads=int(self.args["--threads"]),
+            window_size=int(self.args["--window-size"]),
+            junctions=self.args["--junctions"],
+        )
+
+        generate_log_footer(log_file)
+
+
+class Pairs(AbstractCommand):
+    """Sort and pairs files for faster assess to the data.
+
+    Sort the pairs file using pairtools. Compress them using bgzip. Index them
+    using pairix.
+
+    usage:
+        pairs [--force] [--remove] [--threads=1] <pairsfile>...
+
+    arguments:
+        pairsfile           File(s) containing pairs information.
+
+    options:
+        -F, --force         Write files even if the output files already exists.
+        -r, --remove        Remove the input file at the end to keep only the
+                            sorted, compressed and indexed pairs file.
+        -t, --threads=INT   Numbers of thread to allocate. [Default: 1]
+    """
+
+    def execute(self):
+        # Generate log
+        now = time.strftime("%Y%m%d%H%M%S")
+        log_file = f"metator_pairs_{now}.log"
+        generate_log_header(log_file, cmd="pairs", args=self.args)
+        # Iterates on pairfiles given.
+        pairsfiles = self.args["<pairsfile>"]
+        for pairsfile in pairsfiles:
+            logger.info(f"Processing {pairsfile}...")
+            # Run the sort/compress/index command.
+            _ = mio.sort_pairs_pairtools(
+                pairsfile,
+                threads=int(self.args["--threads"]),
+                remove=self.args["--remove"],
+                force=self.args["--force"],
+            )
+        generate_log_footer(log_file)
+
+
+def generate_log_header(log_path, cmd, args):
+    mtl.set_file_handler(log_path, formatter=logging.Formatter(""))
+    logger.info(f"## MetaTOR: v{__version__} log file")
+    logger.info(f"## date: {time.strftime('%Y-%m-%d %H:%M:%S')}")
+    logger.info(
+        f"## Command used: metator {cmd} {' '.join(f'{key} {value}' for key, value in args.items())}"
+    )
+    logger.info("\n---\n")
+    mtl.set_file_handler(log_path, formatter=mtl.logfile_formatter)
+
+
+def generate_log_footer(log_path):
+    mtl.set_file_handler(log_path, formatter=logging.Formatter(""))
+    logger.info("\n---")
+    logger.info(f"## date: {time.strftime('%Y-%m-%d %H:%M:%S')}")
