@@ -8,15 +8,21 @@ This mdoule contains all core I/O functions:
     - check_fasta_index
     - check_is_fasta
     - check_louvain_cpp
+    - check_pairix
+    - check_pairtools
     - generate_fasta_index
     - generate_temp_dir
     - get_restriction_site
+    - get_pairs_data
     - process_ligation_sites
+    - read_bin_summary
     - read_compressed
+    - read_contig_data
     - read_results_checkm
     - retrieve_fasta
     - sort_pairs
-    - write_checkm_summary
+    - sort_pairs_pairtools
+    - write_bin_summary
 """
 
 import bz2
@@ -26,6 +32,7 @@ import numpy as np
 import os
 import pandas as pd
 import pathlib
+import pypairix
 import re
 import subprocess as sp
 import zipfile
@@ -144,33 +151,76 @@ def check_louvain_cpp(louvain_path):
     # Check convert:
     try:
         convert = sp.check_output(
-            "{0} --help".format(convert), stderr=sp.STDOUT, shell=True
+            f"{convert} --help", stderr=sp.STDOUT, shell=True
         )
     except sp.CalledProcessError:
         logger.error("Cannot find the 'convert' function from Louvain path.")
         return False
+        raise ImportError
 
     # Check louvain:
     try:
         louvain = sp.check_output(
-            "{0} --help".format(louvain), stderr=sp.STDOUT, shell=True
+            f"{louvain} --help", stderr=sp.STDOUT, shell=True
         )
     except sp.CalledProcessError:
         logger.error("Cannot find the 'louvain' function from Louvain path.")
         return False
+        raise ImportError
 
     # Check hierarchy:
     try:
         hierarchy = sp.check_output(
-            "{0} --help".format(hierarchy), stderr=sp.STDOUT, shell=True
+            f"{hierarchy} --help", stderr=sp.STDOUT, shell=True
         )
     except sp.CalledProcessError:
         logger.error("Cannot find the convert_net function from Louvain path.")
         return False
+        raise ImportError
 
     return True
 
 
+def check_pairix():
+    """
+    Function to test if pairix is in the path.
+
+    Returns:
+    --------
+    bool:
+        True if pairix found in the path, False otherwise.
+    """
+    try:
+        pairix = sp.check_output("pairix --help", stderr=sp.STDOUT, shell=True)
+    except sp.CalledProcessError:
+        logger.error(
+            "Cannot find 'pairix' in your path please install it or add it in your path."
+        )
+        raise ImportError
+        return False
+    return True
+
+
+def check_pairtools():
+    """
+    Function to test if pairtools is in the path.
+
+    Returns:
+    --------
+    bool:
+        True if pairtools found in the path, False otherwise.
+    """
+    try:
+        pairix = sp.check_output("pairtools", stderr=sp.STDOUT, shell=True)
+    except sp.CalledProcessError:
+        logger.error(
+            "Cannot find 'pairtools' in your path please install it or add it in your path."
+        )
+        raise ImportError
+        return False
+    return True
+
+
 def generate_fasta_index(fasta, aligner, outdir):
     """Generate fasta index.
 
@@ -188,7 +238,7 @@ def generate_fasta_index(fasta, aligner, outdir):
     str:
         Path to the bowtie2 index build
     """
-    logger.info("Build index from the given fasta.")
+    logger.info("Build index from the given fasta.\n")
     index = join(outdir, "index")
     if aligner == "bowtie2":
         cmd = "bowtie2-build -q {0} {1}".format(fasta, index)
@@ -232,6 +282,40 @@ def generate_temp_dir(path):
     return full_path
 
 
+def get_pairs_data(pairfile, threads=1, remove=False, force=False):
+    """Extract pairs data from pypairix indexed pairs file. If no pypairix
+    indexed found, sort pairs files using pairtools executable.
+
+    Parameters
+    ----------
+    pairfile : str
+        Path to the pairfile to extract data.
+    threads : int
+        Number of threads to use to sort pairs if necessary. [Default: 1]
+    remove : bool
+        If set to true, it will remove the unsorted pair file. [Default: False]
+    force : bool
+        If set overwrite existing files. [Default: False]
+
+    Returns
+    -------
+    str :
+        Path to the sorted and indexed pair file.
+    """
+    # Check if pairix index exists, generate it otherwise.
+    try:
+        pairs_data = pypairix.open(pairfile)
+    except pypairix.PairixError:
+        try:
+            pairfile_sorted = f"{os.path.splitext(pairfile)[0]}_sorted.pairs.gz"
+            pairs_data = pypairix.open(pairfile_sorted)
+        except pypairix.PairixError:
+            logger.warning("No pairix index found. Iterates on the pairs.")
+            pairfile = sort_pairs_pairtools(pairfile, threads, remove, force)
+            pairs_data = pypairix.open(pairfile)
+    return pairs_data
+
+
 def get_restriction_site(enzyme):
     """Function to return a regex which corresponds to all possible restriction
     sites given a set of enzyme.
@@ -275,6 +359,51 @@ def get_restriction_site(enzyme):
     return pattern
 
 
+def micomplete_results_to_dict(micomplete_file):
+    """Read micomplte output file and transfrom it as a dictionnary with the
+    bin name as keys and bin information as values.
+
+    Parameters
+    ----------
+    micomplete_file : str
+        Path to the micomplete output file.
+
+    Returns
+    -------
+    dict
+        Dictionnary of the output of miComplete as values and the bin id as
+        keys.
+    """
+    # Read table.
+    micomplete_summary = pd.read_csv(
+        micomplete_file,
+        sep="\t",
+        comment="#",
+        index_col=0,
+    ).iloc[:, :13]
+
+    # Transform to dictionnary.
+    micomplete_summary = micomplete_summary.to_dict(orient="index")
+
+    return micomplete_summary
+
+
+def read_bin_summary(bin_summary_file):
+    """Read bin summary file from metator pipeline.
+
+    Parameters
+    ----------
+    bin_summary_file : str
+        Path to the bin summary file.
+
+    Returns
+    -------
+    pandas.DataFrame
+        Table from bin summary with bin name as index.
+    """
+    return pd.read_csv(bin_summary_file, sep="\t", index_col=0)
+
+
 def read_compressed(filename):
     """Read compressed file
 
@@ -333,6 +462,24 @@ def read_compressed(filename):
         return open(filename, "r")
 
 
+def read_contig_data(contig_data_file):
+    """Read bin summary file from metator pipeline.
+
+    Parameters
+    ----------
+    contig_data_file : str
+        Path to the contig data file.
+
+    Returns
+    -------
+    pandas.DataFrame
+        Table from contig data with contig name as index.
+    """
+    data = pd.read_csv(contig_data_file, sep="\t")
+    data = data.set_index("Name", drop=False)
+    return data
+
+
 def read_results_checkm(checkm_file, checkm_taxonomy_file):
     """Function to transform the output summary file of checkm into a
     dictionnary.
@@ -410,7 +557,7 @@ def retrieve_fasta(in_file, aligner, tmpdir):
     else:
         if check_fasta_index(in_file, aligner):
             if aligner == "bowtie2":
-                logger.info("Retrieve fasta from bowtie2 index.")
+                logger.info("Retrieve fasta from bowtie2 index.\n")
                 fasta = join(tmpdir, "assembly.fa")
                 cmd = "bowtie2-inspect {0} > {1}".format(in_file, fasta)
                 process = sp.Popen(cmd, shell=True, stdout=sp.PIPE)
@@ -531,13 +678,74 @@ def sort_pairs(in_file, out_file, tmp_dir=None, threads=1, buffer="2G"):
         sort_proc.communicate()
 
 
-def write_checkm_summary(bin_summary, bin_summary_file):
+def sort_pairs_pairtools(pairfile, threads=1, remove=False, force=False):
+    """Sort pairs files using pairtools executable. Pairix only works with
+    compressed pair files. So we use bgzip to compress them.
+
+    Parameters
+    ----------
+    pairfile : str
+        Path to the pairfile to sort, compress and index.
+    threads : int
+        Number of threads to use. [Default: 1]
+    remove : bool
+        If set to true, it will remove the unsorted pair file. [Default: False]
+    force : bool
+        If set overwrite existing files. [Default: False]
+
+    Returns
+    -------
+    str :
+        Path to the sorted and indexed pair file.
+    """
+    # Extract basename of the file.
+    basename = os.path.splitext(pairfile)[0]
+
+    # Test if pairix and pairtools are installed and in the path.
+    _ = check_pairix()
+    _ = check_pairtools()
+
+    # Set the force parameter and delete files or raise an error accodringly.
+    if force:
+        force = " -f"
+        if os.path.isfile(f"{basename}_sorted.pairs"):
+            os.remove(f"{basename}_sorted.pairs")
+    else:
+        force = ""
+        if os.path.isfile(f"{basename}_sorted.pairs") or os.path.isfile(
+            f"{basename}_sorted.pairs.gz"
+        ):
+            logger.error(
+                f"The {basename}_sorted.pairs exists. Do not overwrite existing, use --force to overwrite or use another location."
+            )
+            raise ValueError
+
+    # Sort pairs using pairtools.
+    cmd = f"set -eu ; pairtools sort {pairfile} --nproc {threads} -o {basename}_sorted.pairs"
+    process = sp.Popen(cmd, shell=True)
+    _out, _err = process.communicate()
+    # Compressed pairs.
+    cmd = f"set -eu ; bgzip {basename}_sorted.pairs -@ {threads}{force}"
+    process = sp.Popen(cmd, shell=True)
+    _out, _err = process.communicate()
+    # Indexed pairs.
+    cmd = f"set -eu ;  pairix{force} {basename}_sorted.pairs.gz"
+    process = sp.Popen(cmd, shell=True)
+    _out, _err = process.communicate()
+
+    # Remove original pairfile if remove setup.
+    if remove:
+        os.remove(pairfile)
+    return f"{basename}_sorted.pairs.gz"
+
+
+def write_bin_summary(bin_summary, bin_summary_file):
     """Function to write the bin summary from dictionnary to table text file.
 
     Parameters:
     -----------
     bin_summary : dict
-        Dictionnary with the output of the checkM of the bins.
+        Dictionnary with the output of the miComplete of the bins.
     bin_summary_file : str
         Path to the output file to write the summary informations of the bins.
     """
